@@ -585,7 +585,7 @@ void print_z_stream(const z_stream *s)   // temporary tracing function for #4099
 {
   const unsigned char *byte = (unsigned char *)s;
   for (int i=0; i<sizeof(z_stream); ++i) {
-    DTPRINT("%02x ", byte[i]);
+    DTPRINT("%02x ", *byte++);  // not byte[i] is attempt to avoid valgrind's use-of-uninitialized, #4639, and z_stream={0} now too
   }
   DTPRINT("\n");
 }
@@ -735,7 +735,7 @@ void fwriteMain(fwriteMainArgs args)
     } else {
       int ret1=0, ret2=0;
       if (args.is_gzip) {
-        z_stream stream;
+        z_stream stream = {0};
         if(init_stream(&stream)) {
           free(buff);                                    // # nocov
           STOP(_("Can't allocate gzip stream structure"));  // # nocov
@@ -798,7 +798,7 @@ void fwriteMain(fwriteMainArgs args)
   // compute zbuffSize which is the same for each thread
   size_t zbuffSize = 0;
   if(args.is_gzip){
-    z_stream stream;
+    z_stream stream = {0};
     if(init_stream(&stream))
       STOP(_("Can't allocate gzip stream structure")); // # nocov
     zbuffSize = deflateBound(&stream, buffSize);
@@ -841,7 +841,7 @@ void fwriteMain(fwriteMainArgs args)
 
     void *myzBuff = NULL;
     size_t myzbuffUsed = 0;
-    z_stream mystream;
+    z_stream mystream = {0};
     if (args.is_gzip) {
       myzBuff = zbuffPool + me*zbuffSize;
       if (init_stream(&mystream)) { // this should be thread safe according to zlib documentation
